@@ -13,19 +13,19 @@
 ##############################################################################
 
 # The next line must use double quotes, so release.py recognizes it.
-__version__ = "3.4a0"
+__version__ = "3.6.0"
 
 import sys
-import __builtin__
 
 from persistent import TimeStamp
-from DB import DB
-from transaction import get as get_transaction
+from persistent import list
+from persistent import mapping
 
-# Backward compat for old imports. I don't think TimeStamp should
-# really be in persistent anyway.
+# Backward compat for old imports.
 sys.modules['ZODB.TimeStamp'] = sys.modules['persistent.TimeStamp']
+sys.modules['ZODB.PersistentMapping'] = sys.modules['persistent.mapping']
+sys.modules['ZODB.PersistentList'] = sys.modules['persistent.list']
 
-# TODO Issue deprecation warning if this variant is used?
-__builtin__.get_transaction = get_transaction
-del __builtin__
+del mapping, list, sys
+
+from DB import DB
