@@ -13,7 +13,7 @@
 ##############################################################################
 """Adding implementation tests
 
-$Id: test_adding.py 29143 2005-02-14 22:43:16Z srichter $
+$Id: test_adding.py 38357 2005-09-07 20:14:34Z srichter $
 """
 import unittest
 
@@ -36,7 +36,8 @@ from zope.app.traversing.interfaces import IContainmentRoot
 from zope.app.exception.interfaces import UserError
 from zope.app.publisher.browser import BrowserView
 from zope.app.publisher.interfaces.browser import AddMenu
-from zope.app.publisher.browser.menu import BrowserMenuItem
+from zope.app.publisher.interfaces.browser import IMenuItemType, IBrowserMenu
+from zope.app.publisher.browser.menu import BrowserMenuItem, BrowserMenu
 from zope.app.container.interfaces import IAdding
 from zope.app.container.interfaces import IObjectAddedEvent
 from zope.app.container.interfaces import IContainerNamesContainer
@@ -94,6 +95,12 @@ def defineMenuItem(menuItemType, for_, action, title=u'', extra=None):
     zope.interface.classImplements(newclass, menuItemType)
     ztapi.provideAdapter((for_, IBrowserRequest), menuItemType, newclass, title)
 
+def registerAddMenu():
+  ztapi.provideUtility(IMenuItemType, AddMenu, 'zope.app.container.add')
+  ztapi.provideUtility(IBrowserMenu,
+                       BrowserMenu('zope.app.container.add', u'', u''),
+                       'zope.app.container.add')
+
 
 class Test(PlacelessSetup, unittest.TestCase):
 
@@ -188,10 +195,10 @@ class Test(PlacelessSetup, unittest.TestCase):
         ztapi.browserView(None, '', AbsoluteURL, providing=IAbsoluteURL)
         self.assertRaises(UserError, adding.action, '', 'foo')
         adding.action('Thing', 'foo')
-        self.assertEqual(adding.request.response._headers['location'],
+        self.assertEqual(adding.request.response.getHeader('location'),
                          '/container/+/Thing=foo')
         adding.action('Thing/screen1', 'foo')
-        self.assertEqual(adding.request.response._headers['location'],
+        self.assertEqual(adding.request.response.getHeader('location'),
                          '/container/+/Thing/screen1=foo')
 
     def test_publishTraverse_factory(self):
@@ -206,9 +213,15 @@ class Test(PlacelessSetup, unittest.TestCase):
 def test_constraint_driven_addingInfo():
     """
     >>> setUp()
+    >>> registerAddMenu()
 
     >>> class TestMenu(zope.interface.Interface):
     ...     pass
+    >>> zope.interface.directlyProvides(TestMenu, IMenuItemType)
+
+    >>> ztapi.provideUtility(IMenuItemType, TestMenu, 'TestMenu')
+    >>> ztapi.provideUtility(IBrowserMenu, BrowserMenu('TestMenu', u'', u''),
+    ...                      'TestMenu')
 
     >>> defineMenuItem(TestMenu, IAdding, '', 'item1')
     >>> defineMenuItem(TestMenu, IAdding, '', 'item2')
@@ -252,7 +265,7 @@ def test_constraint_driven_addingInfo():
     >>> items[0]['title']
     'item3'
     
-    >>> adding.menu_id = TestMenu
+    >>> adding.menu_id = 'TestMenu'
     >>> items = adding.addingInfo()
     >>> len(items)
     3
@@ -400,6 +413,7 @@ def test_SingleMenuItem_and_CustomAddView_NonICNC():
     the container contains only a single content object
     
     >>> setUp()
+    >>> registerAddMenu()
     >>> defineMenuItem(AddMenu, IAdding, '', 'item3', extra={'factory': 'f1'})
 
     >>> class F1(object):
@@ -459,6 +473,7 @@ def test_SingleMenuItem_and_NoCustomAddView_NonICNC():
     implement IContainerNamesContainer
     
     >>> setUp()
+    >>> registerAddMenu()
     >>> defineMenuItem(AddMenu, None, '', 'item3', extra={'factory': ''})
     >>> class F1(object):
     ...     pass
@@ -515,6 +530,7 @@ def test_isSingleMenuItem_with_ICNC():
     and the container uses IContainerNamesContaienr
 
     >>> setUp()
+    >>> registerAddMenu()
     >>> defineMenuItem(AddMenu, None, '', 'item3', extra={'factory': ''})
     
     >>> class F1(object):
