@@ -15,7 +15,7 @@
 
 Encapsulation of date/time values
 
-$Id: datetimeutils.py 26316 2004-07-09 15:45:37Z srichter $
+$Id: datetimeutils.py 38178 2005-08-30 21:50:19Z mj $
 """
 import math
 import re
@@ -521,7 +521,7 @@ class DateTimeParser:
         will raise a DateTimeError.
         """
         if not isinstance(arg, StringTypes):
-            raise TypeError, 'Expected a string argument'
+            raise TypeError('Expected a string argument')
 
         if not arg:
             raise SyntaxError(arg)
