@@ -13,15 +13,19 @@
 ##############################################################################
 """Functional doc tests for demo widget implementation
 
-$Id: ftests.py 29331 2005-02-27 23:49:31Z rogerineichen $
+$Id: ftests.py 29574 2005-03-19 18:03:23Z rogerineichen $
 """
 
 import unittest
-from zope.testing import doctest, doctestunit
 from zope.app.testing import functional
 
 def test_suite():
-    return functional.FunctionalDocFileSuite('./help/textwidget.txt')
+    return unittest.TestSuite((
+        functional.FunctionalDocFileSuite('./help/boolwidget.txt'),
+        functional.FunctionalDocFileSuite('./help/intwidget.txt'),
+        functional.FunctionalDocFileSuite('./help/textwidget.txt'),
+        functional.FunctionalDocFileSuite('./help/textareawidget.txt'),
+        ))
 
 if __name__ == '__main__':
     unittest.main(defaultTest='test_suite')
