@@ -13,7 +13,7 @@
 ##############################################################################
 """Code for the toFS.snarf view and its inverse, fromFS.snarf.
 
-$Id: __init__.py 37557 2005-07-29 18:19:31Z benji_york $
+$Id: __init__.py 39064 2005-10-11 18:40:10Z philikon $
 """
 __docformat__ = 'restructuredtext'
 
@@ -31,7 +31,7 @@ from zope.app.fssync import syncer
 from zope.app.fssync.committer import Committer, Checker
 from zope.fssync.metadata import Metadata
 
-from zope.app.i18n import ZopeMessageIDFactory as _
+from zope.app.i18n import ZopeMessageFactory as _
 
 def snarf_dir(response, dirname):
     """Helper to snarf a directory to the response."""
@@ -127,9 +127,8 @@ class SnarfSubmission(BrowserView):
             shutil.rmtree(self.tempdir)
 
     def unsnarf_body(self):
-        fp = self.request.bodyFile
-        fp.seek(0)
-        uns = Unsnarfer(fp)
+        stream = self.request.bodyStream
+        uns = Unsnarfer(stream)
         uns.unsnarf(self.tempdir)
 
     def call_committer(self):
