@@ -16,49 +16,25 @@ ILockInfo implementation.
 
 $Id: $
 """
+import time
+import persistent.mapping
+import zope.interface
+from zope.app.locking.interfaces import ILockInfo
 
-from zope.app.locking.interfaces import ILockInfo, LockingError
-import zope.interface, time
+class LockInfo(persistent.mapping.PersistentMapping):
 
-
-class LockInfo(object):
-    
     zope.interface.implements(ILockInfo)
 
     def __init__(self, target, principal_id, timeout=None):
-        self.target = target
+        # must not store target with security proxy.
+        super(LockInfo, self).__init__()
+        self.__parent__ = self.target = target
         self.principal_id = principal_id
         self.created = time.time()
         self.timeout = timeout
-        self.data = {}
-
-    def get(self, key, default=None):
-        return self.data.get(key, default)
-
-    def keys(self):
-        return self.data.keys()
-
-    def values(self):
-        return self.data.values()
-
-    def items(self):
-        return self.data.items()
-
-    def __getitem__(self, key):
-        return self.data[key]
-
-    def __setitem__(self, key, value):
-        self.data[key] = value
-
-    def __delitem__(self, key):
-        del self.data[key]
-
-    def __contains__(self, key):
-        return self.data.has_key(key)
-
-    def __iter__(self):
-        return iter(self.data)
-
-    def __len__(self):
-        return len(self.data)
 
+    def __repr__(self):
+        return "<%s.%s object at 0x%x>" % (
+            self.__class__.__module__,
+            self.__class__.__name__,
+            id(self))
