@@ -13,7 +13,7 @@
 ##############################################################################
 """Login and Logout screens
 
-$Id: auth.py 30727 2005-06-10 08:54:29Z hdima $
+$Id: auth.py 39064 2005-10-11 18:40:10Z philikon $
 """
 import urllib
 from zope.interface import implements
@@ -24,7 +24,7 @@ from zope.app.security.interfaces import IAuthentication
 from zope.app.security.interfaces import IUnauthenticatedPrincipal
 from zope.app.security.interfaces import ILogout, ILogoutSupported
 from zope.app.pagetemplate import ViewPageTemplateFile
-from zope.app.i18n import ZopeMessageIDFactory as _
+from zope.app.i18n import ZopeMessageFactory as _
 
 
 search_label = _('search-button', 'Search')
