@@ -13,7 +13,7 @@
 ##############################################################################
 """Functions that control how the Zope appserver knits itself together.
 
-$Id: main.py 30643 2005-06-04 13:00:44Z jim $
+$Id: main.py 39539 2005-10-20 15:07:29Z fdrake $
 """
 import logging
 import os
@@ -22,13 +22,9 @@ import time
 
 from zdaemon import zdoptions
 
-import ZODB.interfaces
-
 import ThreadedAsync
 
-from zope import component, interface
-
-import zope.app.appsetup
+import zope.app.appsetup.appsetup
 import zope.app.appsetup.interfaces
 from zope.event import notify
 from zope.server.taskthreads import ThreadedTaskDispatcher
@@ -65,76 +61,12 @@ def main(args=None):
     sys.exit(0)
 
 
-def multi_database(database_factories):
-    """Set up a multi-database from an iterable of database factories
-
-    Return a sequence of databases, and a mapping of from database name to
-    database.
-
-    >>> class DB:
-    ...     def __init__(self, number):
-    ...         self.number = number
-    ...     def __repr__(self):
-    ...         return "DB(%s)" % self.number
-
-    >>> class Factory:
-    ...     def __init__(self, name, number):
-    ...         self.name = name
-    ...         self.number = number
-    ...     def open(self):
-    ...         return DB(self.number)
-
-    >>> s, m = multi_database(
-    ...           [Factory(None, 3), Factory('y', 2), Factory('x', 1)])
-
-    >>> list(s)
-    [DB(3), DB(2), DB(1)]
-
-    >>> [d.database_name for d in s]
-    ['', 'y', 'x']
-
-    >>> [d.databases is m for d in s]
-    [True, True, True]
-
-    >>> items = m.items()
-    >>> items.sort()
-    >>> items
-    [('', DB(3)), ('x', DB(1)), ('y', DB(2))]
-
-    Each of the databases is registered as an IDatabase utility:
-
-    >>> [(component.getUtility(ZODB.interfaces.IDatabase, name) is m[name])
-    ...  for name in m]
-    [True, True, True]
-    
-    """
-
-    databases = {}
-    result = []
-    for factory in database_factories:
-        name = factory.name or ''
-        if name in databases:
-            raise ValueError("Duplicate database name: %r" % name)
-        db = factory.open()
-        db.databases = databases
-        db.database_name = name
-        databases[name] = db
-        # Grrr bug in ZODB. Database doesn't declare that it implements
-        # IDatabase.
-        if not ZODB.interfaces.IDatabase.providedBy(db):
-            interface.directlyProvides(db, ZODB.interfaces.IDatabase)
-        component.provideUtility(db, ZODB.interfaces.IDatabase, name)
-        result.append(db)
-
-    return result, databases
-        
-
 def debug(args=None):
     options = load_options(args)
 
     zope.app.appsetup.config(options.site_definition)
 
-    db = multi_database(options.databases)[0][0]
+    db = zope.app.appsetup.appsetup.multi_database(options.databases)[0][0]
     notify(zope.app.appsetup.interfaces.DatabaseOpened(db))
     return db
 
@@ -166,9 +98,10 @@ def setup(options):
     options.eventlog()
     options.accesslog()
 
-    zope.app.appsetup.config(options.site_definition)
+    zope.app.appsetup.config(options.site_definition,
+                             features=('zserver',))
 
-    db = multi_database(options.databases)[0][0]
+    db = zope.app.appsetup.appsetup.multi_database(options.databases)[0][0]
 
     notify(zope.app.appsetup.interfaces.DatabaseOpened(db))
 
