@@ -15,7 +15,7 @@
 
 See README.txt.
 
-$Id: config.py 29308 2005-02-26 14:13:49Z srichter $
+$Id: config.py 40527 2005-12-04 21:26:50Z efge $
 """
 __docformat__ = 'restructuredtext'
 import os.path
@@ -107,10 +107,13 @@ class ConfigurationContext(object):
         1
         >>> c.resolve('zope.interface') is zope.interface
         1
-        >>> c.resolve('zope.configuration.eek')
+
+        >>> c.resolve('zope.configuration.eek') #doctest: +NORMALIZE_WHITESPACE
         Traceback (most recent call last):
         ...
-        ConfigurationError: Module zope.configuration has no global eek
+        ConfigurationError:
+        ImportError: Module zope.configuration has no global eek
+
         >>> c.resolve('.config.ConfigurationContext')
         Traceback (most recent call last):
         ...
@@ -180,9 +183,11 @@ class ConfigurationContext(object):
         try:
             mod = __import__(mname, *_import_chickens)
         except ImportError, v:
-            raise ConfigurationError, (
-                "Couldn't import %s, %s" % (mname, v)
-                ), sys.exc_info()[2]
+            if sys.exc_info()[2].tb_next is not None:
+                # ImportError was caused deeper
+                raise
+            raise ConfigurationError(
+                "ImportError: Couldn't import %s, %s" % (mname, v))
 
         if not oname:
             # see not mname case above
@@ -200,24 +205,11 @@ class ConfigurationContext(object):
             try:
                 return __import__(mname+'.'+oname, *_import_chickens)
             except ImportError:
-
-                # We need to try to figure out what module the import
-                # error is complaining about.  If the import failed
-                # due to a failure to import some other module
-                # imported by the module we are importing, we want to
-                # know it. Unfortunately, the value of the import
-                # error is just a string error message. :( We can't
-                # pull it apart directly to see what module couldn't
-                # be imported. The only thing we can really do is to
-                # try to "screen scrape" the error message:
-
-                if str(sys.exc_info()[1]).find(oname) < 0:
-                    # There seems to have been an error further down,
-                    # so reraise the exception so as not to hide it.
+                if sys.exc_info()[2].tb_next is not None:
+                    # ImportError was caused deeper
                     raise
-
-                raise ConfigurationError("Module %s has no global %s"
-                                         % (mname, oname))
+                raise ConfigurationError(
+                    "ImportError: Module %s has no global %s" % (mname, oname))
 
     def path(self, filename):
         """
@@ -258,6 +250,7 @@ class ConfigurationContext(object):
                 basepath = os.getcwd()
             else:
                 basepath = os.path.dirname(self.package.__file__)
+                basepath = os.path.abspath(basepath)
             self.basepath = basepath
 
         return os.path.join(basepath, filename)
@@ -369,7 +362,7 @@ class ConfigurationContext(object):
 
         Finally, we can add an order argument to crudely control the order
         of execution:
-        
+
         >>> c.action(None, order=99999)
         >>> c.actions[-1]
         (None, None, (), {}, ('foo.zcml',), '?', 99999)
@@ -609,23 +602,22 @@ class ConfigurationMachine(ConfigurationAdapterRegistry, ConfigurationContext):
 
 
         """
-        for action in resolveConflicts(self.actions):
-            (discriminator, callable, args, kw, includepath, info, order
-             ) = expand_action(*action)
-            if callable is None:
-                continue
-            try:
-                callable(*args, **kw)
-            except:
-                if testing:
-                    raise
-                t, v = sys.exc_info()[:2]
-                v = ConfigurationExecutionError(t, v, info)
-                t = ConfigurationExecutionError
-                raise t, v, sys.exc_info()[2]
-
-        if clear:
-            del self.actions[:]
+        try:
+            for action in resolveConflicts(self.actions):
+                (discriminator, callable, args, kw, includepath, info, order
+                 ) = expand_action(*action)
+                if callable is None:
+                    continue
+                try:
+                    callable(*args, **kw)
+                except:
+                    if testing:
+                        raise
+                    t, v, tb = sys.exc_info()
+                    raise ConfigurationExecutionError(t, v, info), None, tb
+        finally:
+            if clear:
+                del self.actions[:]
 
 
 class ConfigurationExecutionError(ConfigurationError):
@@ -1388,8 +1380,8 @@ def toargs(context, schema, data):
             try:
                 args[str(name)] = field.fromUnicode(s)
             except zope.schema.ValidationError, v:
-                raise ConfigurationError, (
-                    "Invalid value for", n, str(v)), sys.exc_info()[2]
+                raise ConfigurationError(
+                    "Invalid value for", n, str(v)), None, sys.exc_info()[2]
         elif field.required:
             # if the default is valid, we can use that:
             default = field.default
