@@ -15,8 +15,9 @@
 
 An implementation of a TAL expression engine
 
-$Id: tales.py 26559 2004-07-15 21:22:32Z srichter $
+$Id: tales.py 39752 2005-10-30 20:16:09Z srichter $
 """
+__docformat__ = "reStructuredText"
 import re
 
 from zope.interface import implements
@@ -83,7 +84,7 @@ class Iterator(object):
         >>> it = Iterator('foo', {"apple":1, "pear":1, "orange":1}, context)
         >>> it.next()
         True
-        
+
         >>> it = Iterator('foo', {}, context)
         >>> it.next()
         False
@@ -187,7 +188,7 @@ class Iterator(object):
         """
         index = self._nextIndex - 1
         if index < 0:
-            raise TypeError("No iteration position") 
+            raise TypeError("No iteration position")
         return index
 
     def number(self):
@@ -560,13 +561,11 @@ class ExpressionEngine(object):
 
     def registerType(self, name, handler):
         if not _valid_name(name):
-            raise RegistrationError, (
-                'Invalid expression type name "%s".' % name)
+            raise RegistrationError('Invalid expression type name "%s".' % name)
         types = self.types
         if name in types:
-            raise RegistrationError, (
-                'Multiple registrations for Expression type "%s".' %
-                name)
+            raise RegistrationError(
+                'Multiple registrations for Expression type "%s".' % name)
         types[name] = handler
 
     def getTypes(self):
@@ -574,10 +573,10 @@ class ExpressionEngine(object):
 
     def registerBaseName(self, name, object):
         if not _valid_name(name):
-            raise RegistrationError, 'Invalid base name "%s".' % name
+            raise RegistrationError('Invalid base name "%s".' % name)
         base_names = self.base_names
         if name in base_names:
-            raise RegistrationError, (
+            raise RegistrationError(
                 'Multiple registrations for base name "%s".' % name)
         base_names[name] = object
 
@@ -595,8 +594,7 @@ class ExpressionEngine(object):
         try:
             handler = self.types[type]
         except KeyError:
-            raise CompilerError, (
-                'Unrecognized expression type "%s".' % type)
+            raise CompilerError('Unrecognized expression type "%s".' % type)
         return handler(type, expr, self)
 
     def getContext(self, contexts=None, **kwcontexts):
