@@ -25,5 +25,24 @@ SchoolBell libraries
 - schoolbell.app is a calendaring server and web application for Zope 3
 """
 
-from zope.i18n import MessageIDFactory
-SchoolBellMessageID = MessageIDFactory("schoolbell")
+VERSION = "1.2.4"
+
+from zope.i18n import MessageFactory
+SchoolBellMessage = MessageFactory("schoolbell")
+
+# XXX - unquote_uri copied and pasted from schooltool.common when porting
+# patches from the trunk to prevent a schooltool dependency on schoolbell.
+# A schoolbell.common module was not added as this would break the build.
+import urllib
+def unquote_uri(uri):
+    r"""Unquote a URI.
+
+       >>> unquote_uri('/terms/%C5%BEiema')
+       u'/terms/\u017eiema'
+
+       >>> unquote_uri(u'/terms/%C5%BEiema')
+       u'/terms/\u017eiema'
+
+    """
+    return urllib.unquote(str(uri)).decode('UTF-8')
+
