@@ -35,7 +35,7 @@ IPersonContained, IGroupContained and IResourceContained describe persons,
 groups and resources in context of the rest of the application.  Partly it is
 done to avoid a circular dependency between IXxxContained and IXxxContainer.
 
-$Id: interfaces.py 4436 2005-08-02 09:48:51Z tvon $
+$Id: interfaces.py 5751 2006-02-11 13:49:09Z ignas $
 """
 
 import calendar
@@ -52,7 +52,7 @@ from zope.app.annotation.interfaces import IAnnotatable
 
 import pytz
 
-from schoolbell import SchoolBellMessageID as _
+from schoolbell import SchoolBellMessage as _
 from schoolbell.calendar.interfaces import IEditCalendar, ICalendarEvent
 
 
